(in-package 'types)

;;;
;;; This file implements two auxiliary functions, cnstr-union and cnstr-ones
;;; 
 

;;; for efficiency, build the most commonly used lists of ones once and for all.
(defconstant zero-ones nil)
(defconstant one-one (list (type-one)))
(defconstant two-ones (list (type-one) (type-one)))
(defconstant three-ones (list (type-one) (type-one) (type-one)))
(defconstant one-lists (list zero-ones one-one two-ones three-ones))

;;; cnstr-negate takes a constructor c(X1..Xn) and eqns
;;; and builds c( ~X1 1 ... 1) u c(X1 ~X2 1 ... 1) u ... c(X1 ... Xn-1 (type-compl Xn))
;;;             <-------------------- n such terms ------------------------>
;;;
(defun constr-negate (ctype eqns)
  (let ((cid (cons-id ctype))
	(args (cons-args ctype)))
    (constr-negate-aux (build-cons args eqns) cid eqns)))

(defun constr-negate-aux (cs cid eqns)
  (if (null cs)
      (type-zero)
    (type-union 
     (type-cons cid (car cs) eqns)
     (constr-negate-aux (cdr cs) cid eqns)
     eqns
     :disjoint t)))

(defun one-longer-ones (x)
  (cond
   ((null x) one-one)
   ((null (cdr x)) two-ones)
   (t (cons (type-one) x))))

(defun build-cons (arglst eqns)
  (if (null arglst)
      (values nil nil)
    (multiple-value-bind
     (sofar ones)
     (build-cons (cdr arglst) eqns)
     (values
      (cons (cons (type-compl (car arglst) eqns) ones)
	    (mapcar #'(lambda (x) (cons (type-hat (car arglst) eqns) x)) sofar))
      (one-longer-ones ones)))))



;;; (constr-ones (c1   ... cn) (a1 ... an) ) constructs the following sequence
;;; c1(1,...1) u .... u cn(1,....1)
;;; ai specifies the arity of the constructor ci.
;;; constructors are assumed to be distinct.

(defun i-ones (i) 
  (if (<= i 3) 
      (nth i one-lists)
    (make-list i :initial-element (type-one))))

(defun constr-ones (arglst arity-list)
  (if (null arglst)
      (type-zero)
    (mk-union
     (mk-cons (car arglst) (i-ones (car arity-list)))
     (constr-ones (cdr arglst) (cdr arity-list))
     :disjoint t)))
