(in-package 'types)

;;;
;;; type_constraints :: type * type * eqns -> set of constraints {ai <= X, X <= ai} where tlv(X) <= {a1...ai-1}
;;;
;;; See constraints-adt.lisp : for some fns/data-structures for manipulating constraints.
;;;

;;; The following 2-D array says which constraints testing function should be applied.
;;; If testing t1 <= t2 and t1 has type i and t2 has type j, then apply
;;; the function in position (i,j).  Indexing is by rows and then columns.  
;;;
(defconstant *constraintsarray*
  (make-array 
   '(11 11)
   :initial-contents
(list 
;;                          union  intersection  cons   function  variable   bound var     not     dependent   value
;;        0         1         2        3         4          5         6          7         8          9         10
(list #'no-c    #'no-c  #'no-c    #'no-c    #'no-c      #'no-c     #'no-c     #'no-c     #'no-c     #'c-err   #'no-c      );; 0
(list #'c-err   #'no-c  #'c-un-r  #'c-in-r  #'c-err     #'c-err    #'mk-v-r   #'mk-v-r   #'c-err    #'c-err   #'c-val-r   );; 1
(list #'c-un-l  #'no-c  #'c-un-l  #'c-un-l  #'c-un-l    #'c-un-l   #'c-un-lT  #'c-un-lT  #'c-un-l   #'c-err   #'c-un-l    );; 2 
(list #'c-in-l  #'no-c  #'c-in-l  #'c-in-r  #'c-in-l    #'c-in-l   #'c-in-l-v #'c-in-l-v #'c-in-l   #'c-err   #'c-val-rTi );; 3 
(list #'c-zero  #'no-c  #'c-un-r  #'c-in-r  #'c-c       #'c-err    #'mk-v-r   #'mk-v-r   #'c-n      #'c-err   #'c-val-r   );; 4 
(list #'c-err   #'no-c  #'c-un-r  #'c-in-r  #'c-err     #'fun-fun  #'mk-v-r   #'mk-v-r   #'c-err    #'c-err   #'c-val-r   );; 5  
(list #'mk-v-l  #'no-c  #'c-un-rT #'c-in-rT #'mk-v-l    #'mk-v-l   #'v-v      #'mk-v-l   #'mk-v-l   #'c-err   #'c-val-rT  );; 6
(list #'mk-v-l  #'no-c  #'c-un-rT #'c-in-rT #'mk-v-l    #'mk-v-l   #'mk-v-r   #'v-v      #'mk-v-l   #'c-err   #'c-val-rT  );; 7
(list #'c-err   #'no-c  #'c-un-r  #'c-in-r  #'c-err     #'c-err    #'mk-v-r   #'mk-v-r   #'n-n      #'c-err   #'c-val-r   );; 8
(list #'c-dep-z #'no-c  #'c-un-r  #'c-in-r  #'c-dep-l   #'c-dep-l  #'c-dep-lT #'c-dep-lT #'c-dep-l  #'c-err   #'c-val-r   );; 9
(list #'c-val-l #'no-c  #'c-un-r  #'c-in-r  #'c-val-l   #'c-val-l  #'c-val-lT #'c-val-lT #'c-val-l  #'c-err   #'c-val-l   );; 10
)))



;;;
;;; The main routine just applies the appropriate function.
;;; Optimization: if the t1 <= t2 in all substitutions,
;;; then there are no constraints.
;;;
(defun type-constraints-aux (t1 t2 eqns constraints)
 (if (type-<= t1 t2 eqns)
     constraints
   (funcall (aref *constraintsarray* (type-type t1) (type-type t2)) t1 t2 eqns constraints)))

;;;
;;; The top-level function is type-constraints.  It just starts out the
;;; computation in the given environment of eqns.
;;;
(defun type-constraints (t1 t2 eqns )
  (type-constraints-aux t1  t2 eqns (constraints-empty)))
  	                
