(in-package 'types)

;;;
;;; type-disjoint     : type * type * eqns * assumptions -> ()fn -> True/False

;;; This file is the continuation of disjoint_fns.lisp                  
;;; This is where the functions defined in disjoint_fns.lisp get called 
;;;


;;;
;;; The following 2-D array says which disjointness function should be applied.
;;; If we are testing t1 disj t2, t1 has type i and t2 has type j, then apply
;;; the function in position (i,j).  Indexing is by rows and then columns.  
;;; The array is symmetric
;;;
(defconstant *disjointarray*
  (make-array 
   '(11 11)
   :initial-contents
(list 
;;                         union  intersection  cons   function  variable   bound var     not     dependent   value
;;        0         1         2        3         4          5         6          7         8          9         10
(list #'isdisj  #'isdisj  #'isdisj  #'isdisj  #'isdisj   #'isdisj   #'isdisj  #'isdisj  #'isdisj   #'isdisj  #'isdisj  );; 0
(list #'isdisj  #'notdisj #'notdisj #'notdisj #'notdisj  #'notdisj  #'notdisj #'notdisj #'notdisj  #'notdisj #'notdisj );; 1
(list #'isdisj  #'notdisj #'udisj   #'udisj   #'udisj    #'udisj    #'notdisj   #'udisj   #'udisj    #'udisj   #'udisj   );; 2 
(list #'isdisj  #'notdisj #'disju   #'indisj  #'indisj   #'indisj   #'notdisj  #'indisj  #'indisj   #'indisj  #'indisj  );; 3 
(list #'isdisj  #'notdisj #'disju   #'disjin  #'consdisj #'isdisj   #'notdisj  #'disjbv  #'cndisj   #'rdepdisj #'rvaldisj );; 4 
(list #'isdisj  #'notdisj #'disju   #'disjin  #'isdisj   #'notdisj  #'notdisj  #'disjbv  #'isdisj   #'rdepdisj #'rvaldisj );; 5  
(list #'isdisj  #'notdisj #'notdisj #'notdisj #'notdisj #'notdisj   #'notdisj  #'notdisj #'notdisj  #'notdisj #'notdisj  );; 6
(list #'isdisj  #'notdisj #'disju   #'disjin  #'bvdisj   #'bvdisj   #'notdisj  #'bvdisj  #'bvdisj   #'bvdisj  #'bvdisj   );; 7
(list #'isdisj  #'notdisj #'disju   #'disjin  #'ncdisj   #'isdisj   #'notdisj  #'disjbv  #'notdisj  #'rdepdisj #'rvaldisj);; 8
(list #'isdisj  #'notdisj #'disju   #'disjin  #'ldepdisj #'ldepdisj #'notdisj  #'disjbv  #'ldepdisj #'ldepdisj #'ldepdisj);; 9
(list #'isdisj  #'notdisj #'disju   #'disjin  #'lvaldisj #'lvaldisj #'notdisj #'disjbv  #'lvaldisj #'rdepdisj #'lvaldisj);; 10
)))



;;;
;;; The main routine just applies the appropriate function.
;;;	      
(defun type-disjoint-aux (t1 t2 eqns as)
  (funcall (aref *disjointarray* (type-type t1) (type-type t2)) t1 t2 eqns as))

;;;
;;; The top-level function is type-<=.  It just starts out the
;;; computation in the empty environment of assumptions.
;;;
(defun type-disjoint (t1 t2 Eqns)
  (type-disjoint-aux t1 t2 Eqns (assump-empty)))  
                                              
