(in-package 'types)

;;;
;;; type-free-vars-reachable : type * eqns * assumptions -> t/f (t if there is a free variable reachable from
;;; the type (reachable ==> transitive closure over bv-eqns)
;;; Assumptions: should be threaded but aren't (because its hard to do so in lisp)
;;; The algorithm still terminates but may be inefficient.
;;;
;;; 1/14/93  tkl created
 
;;; 0. E,A |- free-vars-reachable(0) = FALSE
;;;

(defun free-vars-reachable-zero (typ eqns as)
  (declare (ignore typ eqns as))
  nil)

;;; 1. E,A |- free-vars-reachable(1) = FALSE
;;;

(defun free-vars-reachable-one (typ eqns as)
  (declare (ignore typ eqns as))
  nil)

;;;     E,A |- free-vars-reachable(X) = L1;  E,A |- free-vars-reachable(Y) = L2
;;; 2. ----------------------------------------------------------
;;;     E,A |- free-vars-reachable(X | Y) = L1 OR L2
;;;

(defun free-vars-reachable-union (typ eqns as)
  (or (type-free-vars-reachable-aux (type-t1 typ) eqns as)
      (type-free-vars-reachable-aux (type-t2 typ) eqns as)))

;;;    E,A |- free-vars-reachable(X) = L1;  E,A |- free-vars-reachable(Y)= L2
;;; 3. ------------------------------------------------------------
;;;    E,A |- free-vars-reachable(X * Y) = L1 OR L2
;;;

(defun free-vars-reachable-inter (typ eqns as)
  (or (type-free-vars-reachable-aux (type-t1 typ) eqns as)
      (type-free-vars-reachable-aux (type-t2 typ) eqns as)))

;;;    for all i 1<=i<=n, E,A |- free-vars-reachable(Xi) = Li
;;; 4. -------------------------------------------------
;;;    E,A |- free-vars-reachable(c(X1,....Xn)) = OR_i Li
;;;

(defun free-vars-reachable-cons (typ eqns as)
  (some #'(lambda (x) (type-free-vars-reachable-aux x eqns as))
	(cons-args typ)))

;;;   for all i 1 <= i <= 3, E,A |- free-vars-reachable(Xi) = Li
;;; 5. --------------------------------------------------
;;;    E,A |- free-vars-reachable(X1 -> X2 + X3) = OR_i Li
;;;

(defun free-vars-reachable-func (typ eqns as)
  (or (type-free-vars-reachable-aux (type-t1 typ) eqns as)
      (type-free-vars-reachable-aux (type-t2 typ) eqns as)
      (type-free-vars-reachable-aux (type-t3 typ) eqns as)))

;;; 6. E,A |- free-vars-reachable(a) = TRUE if a is a free variable

(defun free-vars-reachable-var (typ eqns as)
  (declare (ignore typ eqns as))
  t)
;;; 7a. E, A union {a} |- free-vars-reachable(a) = {}   a is a bound variable
;;;
;;;    (a = rhs) in E; a in A;  E |- free-vars-reachable(rhs) = L
;;; 7b -----------------------------------------------------  a is a bound variable
;;;    E,A |- free-vars-reachable(a) =  L
;;; NOTE : if (a = rhs) not-in E then signal ERROR
;;;

(defun free-vars-reachable-bvar (typ eqns as)
  (cond ( (assump-has typ as) nil)
	( (null (bv-def typ eqns)) (break "Bound Variable Equation not found" typ eqns))
	(t (type-free-vars-reachable-aux (bv-def typ eqns)
					      eqns
					      (assump-add typ t as)))))


;;; 8. E, A |- free-vars-reachable(not(...)) = FALSE
;;;

(defun free-vars-reachable-not (typ eqns as)
  (declare (ignore typ eqns as))
  nil)

;;;    E, A |- free-vars-reachable(A1) = L1 free-vars-reachable(A2) = L2
;;; 9. ------------------------------------------------------
;;;    E, A |- free-vars-reachable(A1?A2) = L1 OR L2
;;;

(defun free-vars-reachable-dep (typ eqns as)
  (or (type-free-vars-reachable-aux (type-t1 typ) eqns as)
      (type-free-vars-reachable-aux (type-t2 typ) eqns as)))


;;;     E, A |- free-vars-reachable(A) = L
;;; 10. -----------------------------------
;;;     E, A |- free-vars-reachable(val(v,A)) = L
;;;

(defun free-vars-reachable-value (typ eqns as)
  (type-free-vars-reachable-aux (value-typ typ) eqns as))


;;; 11. E, A |- free-vars-reachable (for-all x. A) = ERROR

(defun free-vars-reachable-quant (typ eqns as)
  (declare (ignore eqns as))
  (break "~% The function type-free-vars-reachable should NOT have been called on the QUANT type ~a" typ))

;;;     E, A |- free-vars-reachable(A) = L
;;; 12. ----------------------------------------
;;;     E, A |- free-vars-reachable (lambda x. A) = L

(defun free-vars-reachable-lamb (typ eqns as)
  (declare (ignore eqns as))
  (break "~% The function type-free-vars-reachable should NOT have been called on the LAMBDA type ~a" typ))

