(in-package 'types)

;;;
;;; This file is the continuation of free_vars_reachable_fns.lisp
;;; This is where the functions defined in free_vars_reachable_fns.lisp get called 
;;;
;;; type-free-vars-reachable : type * eqns * assumptions -> T/F
;;; t if there is a free variable reachable from the type
;;; (reachable ==> transitive closure over bv-eqns)
;;;
;;; Reachability is a transitive closure computation.
;;;
;;; The assumptions are needed to ensure termination.
;;;


;;;
;;; The main lookup table
;;;
(defconstant *free-vars-reachable-array*
  (make-array '(13)
	      :initial-contents
	      (list #'free-vars-reachable-zero #'free-vars-reachable-one #'free-vars-reachable-union 
		    #'free-vars-reachable-inter 
		    #'free-vars-reachable-cons #'free-vars-reachable-func #'free-vars-reachable-var 
		    #'free-vars-reachable-bvar 
		    #'free-vars-reachable-not #'free-vars-reachable-dep #'free-vars-reachable-value 
		    #'free-vars-reachable-quant #'free-vars-reachable-lamb
)))
	      
;;; The auxiliary free-vars-reachable function : applies the appropriate function

(defun type-free-vars-reachable-aux (typ eqns as)
  (funcall (aref *free-vars-reachable-array* (type-type typ)) typ eqns as))
  

;;; The top level free-vars-reachable function. Calls the type-free-vars-reachable-aux function

(defun type-free-vars-reachable (typ eqns)
  (type-free-vars-reachable-aux typ eqns (assump-empty)))

;;; NOTE: The top-level call starts with no bound variables having been visited
;;; Hence the initial assumptions are empty.
