(in-package 'types)

;;;
;;; This file is the continuation of hom_fns.lisp
;;; This is where the functions defined in hom_fns.lisp get called 
;;;
;;; type-substitute : type * (var -.> type | :no-change) * eqns -> type 
;;; 

;;;
;;; The main lookup table

(defconstant *homarray*
  (make-array '(13)
	      :initial-contents
	      (list #'hom-zero #'hom-one #'hom-union #'hom-inter #'hom-cons #'hom-func 
		    #'hom-var #'hom-bvar #'hom-not #'hom-dep #'hom-value #'hom-quant #'hom-lambda )))
	      
;;; The auxiliary substituting function : applies the appropriate function.

;;;
;;; h is the homomorphism function
;;; p is a predicate that determines whether the homomorphism should be
;;; applied to the type
;;;
(defun type-hom-aux (typ h p eqns) 
  (if (funcall p typ h p eqns)
      (funcall (aref *homarray* (type-type typ)) typ h p eqns)
    :no-change))

;;; The top level homomorphishm function function. Calls the type-hom-aux function
;;; Also keeps track of the number of top level calls. 
;;;
(defun type-hom (typ h p eqns)                
  (let ((result (type-hom-aux typ h p eqns)))
    (if (eql :no-change result) 
	(values typ :no-change)       
      (values result :change))))

