(in-package 'types)

;;; type-inter : Eqns * Type * Type -> Type
;;;
;;; This file logically belongs with inter_fns.lisp, but has to be physically
;;; separate to allow lisp to define things in the right order.
;;;


;;; The  flag *do-inter-opt* is used to determine whether or not the initial optimization checks should
;;; be performed in type-inter-aux.

(defvar *do-inter-opt* t)
(defun inter-on() (setq *do-inter-opt* t))
(defun inter-off() (setq *do-inter-opt* nil))

;;;
;;; The following 2-D array says which intersection function should be applied.
;;; When testing t1 * t2, if t1 has type i and t2 has type j, then apply
;;; the function in position (i,j).  Indexing is by rows and then columns.  
;;;
(defconstant *interarray*
  (make-array 
   '(11 11)
   :initial-contents
(list 
;;                           union  intersection   cons    function     variable      bound var    not     dependent   value
;;        0         1          2          3         4          5           6             7         8          9         10
(list #'type-zero #'type-zero #'type-zero #'type-zero #'type-zero #'type-zero #'type-zero #'type-zero #'type-zero #'type-zero 
      #'type-zero);; 0
(list #'type-zero #'type-one  #'proj-t2 #'proj-t2 #'proj-t2   #'proj-t2   #'proj-t2 #'proj-t2 #'proj-t2  #'proj-t2 #'proj-t2 );; 1
(list #'type-zero #'proj-t1   #'un-l   #'un-l     #'un-l      #'un-l      #'un-l    #'un-l    #'un-l     #'un-l    #'un-l)    ;; 2 
(list #'type-zero #'proj-t1   #'un-r   #'in-l     #'in-l      #'in-l      #'in-l    #'in-l    #'in-l     #'in-l    #'in-l )   ;; 3 
(list #'type-zero #'proj-t1   #'un-r   #'in-r     #'cons-cons #'type-zero #'var-r   #'bv-r    #'cons-not #'dep-r   #'val-r)   ;; 4 
(list #'type-zero #'proj-t1   #'un-r   #'in-r     #'type-zero #'mk-fn*    #'var-r   #'fn&bv   #'type-zero #'dep-r  #'val-r )  ;; 5  
(list #'type-zero #'proj-t1   #'un-r   #'in-r     #'var-l     #'var-l     #'var-l   #'v&bv    #'var-l    #'var-l   #'var-l)   ;; 6
(list #'type-zero #'proj-t1   #'un-r   #'in-r     #'bv-l      #'bv&fn     #'bv&v    #'bv-l    #'bv-l     #'bv-l    #'bv-l  )  ;; 7
(list #'type-zero #'proj-t1   #'un-r   #'in-r     #'not-cons  #'type-zero #'var-r   #'bv-r    #'not-not  #'dep-r   #'val-r )  ;; 8
(list #'type-zero #'proj-t1   #'un-r   #'in-r     #'dep-l     #'dep-l     #'var-r   #'bv-r    #'dep-l    #'dep-l   #'dep-l )  ;; 9
(list #'type-zero #'proj-t1   #'un-r   #'in-r     #'val-l     #'val-l     #'var-r   #'bv-r    #'val-l    #'val-l   #'val-l )  ;; 10
)))



;;;
;;; The main routine just applies the appropriate function 
;;; and updates statistics if required.
;;;	      
(defun type-inter-aux (t1 t2 eqns )
  (if *do-inter-opt*
      (cond
       ( (type-<= t1 t2 eqns) t1)
       ( (type-<= t2 t1 eqns) t2)
       ( (type-disjoint t1 t2 eqns) (type-zero))
       ( t (funcall (aref *interarray* (type-type t1) (type-type t2)) t1 t2 eqns)))
    (funcall (aref *interarray* (type-type t1) (type-type t2)) t1 t2 eqns)))

;;;
;;; The top-level function is type-inter.  It just starts out the computation.
;;;
(defun type-inter (t1 t2 eqns)
   (type-inter-aux t1 t2 eqns))


