(in-package 'types)

;;;
;;; Implementation of lookup tables.
;;;
;;; For now (and maybe forever) these are Lisp hash tables.
;;;

(defun mk-lookup-table ()
  (make-hash-table :test #'equal))

(define-function 'table-lookup #'gethash)

(defun table-add (key value table)
  (setf (gethash key table) value)
  table)

(defun table-clear (table)
  (clrhash table)
  table)

;;;
;;; integer tables are just arrays
;;;

(defun mk-lookup-itable (i &optional (init nil))
  (make-array (1+ i) :adjustable t :initial-element init))

(defun itable-add (key value table)
  (setf (aref table key) value)
  table)

(defun itable-lookup (i tbl)
  (aref tbl i))

(defun itable-adjust (i tbl &optional (init nil))
  (adjust-array tbl i :initial-element init))

;;;
;;; maps a function f over the table.
;;; f should have functionality  key x value -> something.
;;; Call only for side-effect; returns nil.
;;;

(defun map-table (f tbl)
  (if (arrayp tbl)
      (map-itable f tbl)
    (map-htable f tbl)))

(defun map-htable (f tbl) (maphash f tbl))

(defun map-itable (f tbl) 
  (dotimes (i (length tbl))
	   (if (not (null (aref tbl i)))
	       (funcall f i (aref tbl i)))))
;;;
;;; copies a lookup-table (does a hash-table-copy)
;;;
;;; Ed's code:  makes a copy of the hash table.
;;; the function keyf is applied to all the keys
;;; the function valuef is applied to all the values
;;; the default value for either of these implies the identity function.
;;;

(defun copy-lookup-table (old-table 
			  &key 
			  (keyf #'(lambda (x) x)) ; do nothing 
			  (valuef #'(lambda (x) x)) ; do nothing 
			  )
  (declare (type hash-table old-table)
	   (type function keyf valuef))
  (let ((new-table (make-hash-table :test #'equal)))
    (maphash
     #'(lambda (old-key old-val)
	 (let ((new-key (funcall keyf old-key))
	       (new-val (funcall valuef old-val)))
	   (setf (gethash new-key new-table) new-val)
	  ))
     old-table)
    new-table))

(defun copy-lookup-itable (old-table &optional (init nil))
  (let ((new-table (make-array (array-dimensions old-table)
			       :initial-element init)))
    (dotimes (i (first (array-dimensions old-table)) new-table)
	     (setf (aref new-table i) (aref old-table i)))))
