(in-package 'types)

;;;
;;; A reasonable pretty-printing facility for types.
;;;

;;;
;;; These functions "inline" as many bound variable definitions
;;; as possible to make types more readable.
;;;


(defun pp-zero (typ eqns sofar) 
  (declare (ignore eqns sofar))
  typ)

(defun pp-one (typ eqns sofar) 
  (declare (ignore eqns sofar))
  typ)

(defun pp-union (typ eqns sofar)
  (type-union
   (pp-aux (type-t1 typ) eqns sofar)
   (pp-aux (type-t2 typ) eqns sofar)
   eqns
   :disjoint (is-rtype typ)
   :nosubset (is-nsunion typ)))

(defun pp-inter (typ eqns sofar)
  (type-inter
   (pp-aux (type-t1 typ) eqns sofar)
   (pp-aux (type-t2 typ) eqns sofar)
   eqns))

(defun pp-cons (typ eqns sofar)
  (type-cons
   (cons-id typ)
   (mapcar #'(lambda (x) (pp-aux x eqns sofar))
	   (cons-args typ))
   eqns))

(defun pp-func (typ eqns sofar)
  (type-func
   (pp-aux (type-t1 typ) eqns sofar)
   (pp-aux (type-t2 typ) eqns sofar)
   (pp-aux (type-t3 typ) eqns sofar)
   eqns))

(defun pp-dep (typ eqns sofar)
  (type-dep
   (pp-aux (type-t1 typ) eqns sofar)
   (pp-aux (type-t2 typ) eqns sofar)
   eqns))

(defun pp-var (var eqns sofar)
  (declare (ignore eqns sofar))
  var)

;;;
;;; The only interesting case is a bound variable.
;;; If a bound variable is not reachable from itself,
;;; then we can inline it.
;;;
;;; The argument sofar could also be used to see what types have already
;;; been inlined, but we haven't found another strategy that prints
;;; more readable types.
;;;
(defun pp-bvar (var eqns sofar)
  (cond ((null (bv-def var eqns)) var)
	((not (member var (type-vars-reachable (bv-def var eqns) eqns) :test #'equalp))
	 (pp-aux (bv-def var eqns) eqns (cons var sofar)))
	(t var)))

(defun pp-value (typ eqns sofar)
  (type-value
   (value-val typ)
   (pp-aux (value-typ typ) eqns sofar)
   eqns))

(defun pp-not (typ eqns sofar)
  (declare (ignore eqns sofar))
  typ)

(defun pp-quant (typ eqns sofar)
  (type-quant
   (quant-vars typ)
   (pp-aux (quant-typ typ) eqns sofar)
   eqns))

(defun pp-lambda (typ eqns sofar)
  (type-lambda
   (lambda-vars typ)
   (pp-aux (lambda-typ typ) eqns sofar)
   eqns))


   
