(in-package 'types)

(defconstant *pp-array*
  (make-array '(13)
	      :initial-contents
	      (list #'pp-zero #'pp-one #'pp-union #'pp-inter 
		    #'pp-cons #'pp-func #'pp-var #'pp-bvar 
		    #'pp-not #'pp-dep #'pp-value #'pp-quant #'pp-lambda)))

(defun pp-aux (typ eqns sofar) 
  (funcall (aref *pp-array* (type-type typ)) typ eqns sofar))
  
(defun pp (typ eqns)
  (pp-aux typ eqns nil))

(defvar alphabet `(#\a #\b #\c #\d #\e #\f #\g #\h #\i #\j #\k #\l #\m
		   #\n #\o #\p #\q #\r #\s #\t #\u #\v #\w #\x #\y #\z))

(defun var-to-str (v)
  (if (is-var v) (format nil "fv_~s" (var-id v))
    (format nil "bv_~s" (bvar-id v))))

(defun alpha-subst (vlist)
;  (let ((ziplist (mapcar #'cons vlist alphabet)))
  (let ((ziplist (mapcar #'cons vlist (mapcar #'var-to-str vlist))))
    (substitution-list (mapcar #'car ziplist) (mapcar #'cdr ziplist))))


(defun ppt (typ eqns &optional (str t))
  (let* ((s (ptools::default-output-stream str))
	 (newtyp (pp typ eqns))
	 (vlist nil)
	 (deflist nil)
	 (fvlist nil)
	 (lowers nil)
	 (uppers nil))
    ;;
    ;; Collect together all bound variables reachable from the
    ;; type that require recursive definitions.
    ;; Also collect together all free variables and their upper
    ;; and lower bounds that are reachable.
    ;;
    (do* ((todobv (type-all-bvars newtyp eqns))
	  (todofv (type-all-fvars newtyp eqns)))
	 ((and (null todobv) (null todofv))
	  ())
	 (when (not (null todobv))
	       (let ((next (pop todobv)))
		 (when (not (member next vlist :test #'equal))
		       (let ((newtyp (pp (bv-def next eqns) eqns)))
			 (push next vlist)
			 (push newtyp deflist)
			 (setq todofv 
			       (append todofv (type-all-fvars newtyp eqns)))
			 (setq todobv 
			       (append todobv (type-all-bvars newtyp eqns)))))))

	 (when (not (null todofv))
	       (let ((next (pop todofv)))
		 (when (not (member next fvlist :test #'equal))
		       (let ((u (pp (constr-upper-bound next eqns) eqns))
			     (l (pp (constr-lower-bound next  eqns) eqns)))
			 (when (not (and (type-<= (type-one) u eqns)
					 (type-<= l (type-zero) eqns)))
			       (push u uppers)
			       (push l lowers)
			       (push next fvlist)
			       (setq todofv 
				     (append todofv 
					     (type-all-fvars u eqns) 
					     (type-all-fvars l eqns)))
			       (setq todobv 
				     (append todobv 
					     (type-all-bvars u eqns) 
					     (type-all-bvars l eqns))))))))
	 )
    (let ((subst (alpha-subst (type-vars-reachable newtyp eqns :fv-expand t))))
      (pt newtyp eqns s	:substn subst)
      (if (or (not (null fvlist)) 
	      (not (null vlist))) 
	  (format s "~%where~%"))
      (mapc #'(lambda (v def)
		(format s "  ~a = ~a~%" 
			(pt v eqns nil :substn subst)
			(pt def eqns nil :substn subst)))
	    vlist deflist)
      (mapc #'(lambda (v u l)
		(format s "  ~a <= ~a <= ~a~%" 
			(pt l eqns nil :substn subst)			  
			(pt v eqns nil :substn subst)
			(pt u eqns nil :substn subst)))
	    fvlist uppers lowers))
      
      (if (null str)
	  (get-output-stream-string s))))



