(in-package 'types)

;;;
;;; This file is the continuation of printer_fns.lisp.
;;; The division  into two files is needed to make lisp compiler work.
;;;

(defconstant *printarray*
  (make-array '(13)
	      :initial-contents
	      (list #'print-zero #'print-one #'print-union #'print-inter #'print-cons #'print-func 
		    #'print-var #'print-bvar #'print-not #'print-dep #'print-value #'print-quant #'print-lambda)))
	      
(defun print-type (s p typ tbl &key substn)
  (funcall (aref *printarray* (type-type typ)) s p typ tbl :substn substn))
  

;;;
;;; If typ is null, then "undefined" is printed as the type.
;;;
(defun pt (typ tbl &optional (s t) &key substn)
  (let ((str (ptools::default-output-stream s)))
    (if (null typ)
	(format str " *undefined* ")
        (print-type str nil typ tbl :substn substn))
      (if (null s) (get-output-stream-string str))))

;;;
;;; Function for printing an equation.
;;;
(defun print-equation (lhs rhs tbl 
			    &optional (str t) 
			    &key (fmt "~a = ~a~%") substn)
  (format str fmt 
	  (pt lhs tbl nil :substn substn)
	  (pt rhs tbl nil :substn substn)))

;;;
;;; Function printing upper and lower bounds on a type (presumably, a variable).
;;;
(defun print-L&U (l v u tbl 
		    &optional (str t) 
		    &key (fmt "~a <= ~a <= ~a~%") substn)
  (format str fmt 
	  (pt l tbl nil :substn substn)
	  (pt v tbl nil :substn substn)
	  (pt u tbl nil :substn substn)))



;;;
;;; big-pt prints types as follows:
;;;
;;; (pt typ eqns)  WHERE 
;;;    bv1 = (pt rhs1)
;;;    .........
;;;    bvn = (pt rhsn)
;;;
;;;    L1 <= v1 <= U1
;;;    ...
;;;    Lm <= vm <= Um
;;;
;;;  the bvi are the reachable bound variables in the type
;;;  the vi   "        "       free     "          "
;;;    

(defun big-pt (typ eqns &optional (s t)&key substn)
  (let* ((vlist (sort (type-vars-reachable typ eqns :fv-expand t) #'var-<=))
	 (bv-list (remove-if-not #'is-bvar vlist))
	 (fv-list (remove-if-not #'is-var vlist)))
    (pt typ eqns s :substn substn)
    (format s "       WHERE ~%~%")
    (mapc #'(lambda (x) 
	      (print-equation x (bv-def x eqns) eqns s :substn substn))
	  bv-list)
    (format s "~%")
    (mapc #'(lambda (v)
	      (print-L&U (constr-lower-bound v eqns)
			 v
			 (constr-upper-bound v eqns)
			 eqns s
			 :substn substn))
	  fv-list) 
    t))
