(in-package 'types)

;;;
;;; This file is the continuation of new_reachable_fns.lisp
;;; This is where the functions defined in new_reachable_fns.lisp get called 
;;;
;;; type-vars-reachable : type * ans * eqns * assumptions * (type * ans * eqns -> ans) -> ans
;;;
;;; Used to implement various variables reachability analyses.


;;;
;;; The main lookup table

(defconstant *reachable-array*
  (make-array '(13)
	      :initial-contents
	      (list #'vars-reachable-id #'vars-reachable-id #'vars-reachable-union #'vars-reachable-inter 
		    #'vars-reachable-cons #'vars-reachable-func #'vars-reachable-var #'vars-reachable-bvar 
		    #'vars-reachable-id #'vars-reachable-dep #'vars-reachable-value 
		    #'vars-reachable-quant #'vars-reachable-lamb
)))
	      
;;; The auxiliary vars-reachable function : applies the appropriate function.

(defun type-vars-reachable-aux (typ eqns as fn fv-expand bv-expand) 
  (funcall (aref *reachable-array* (type-type typ)) 
	   typ 
	   eqns 
	   as 
	   fn
	   fv-expand 
	   bv-expand))
  

;;; The top level vars-reachable function. Calls the type-vars-reachable-aux function

(defun type-vars-reachable-main (typ eqns fn &key (fv-expand nil) (bv-expand nil) (initial nil))
 (type-vars-reachable-aux typ 
			  eqns 
			  initial
			  fn
			  fv-expand 
			  bv-expand))

;;;
;;; Variables recursively reachable from a type.
;;;

(defun type-vars-reachable (typ eqns &key (fv-expand nil))
  (type-vars-reachable-main typ
			    eqns
			    #'(lambda (typ eqns as) 
				(declare (ignore eqns))
				(cons typ as))
			    :fv-expand fv-expand
			    :bv-expand t))


;;;
;;; Variables non-recursively reachable from a type
;;;


(defun type-all-bvars (typ eqns)
  (type-vars-reachable-main typ
			    eqns
			    #'(lambda (typ eqns vs)
				(declare (ignore eqns))
				(if (is-bvar typ)
				    (cons typ vs)
				    vs))))

(defun type-all-fvars (typ eqns)
  (type-vars-reachable-main typ
			    eqns
			    #'(lambda (typ eqns vs)
				(declare (ignore eqns))
				(if (is-var typ)
				    (cons typ vs)
				    vs))))

(defun type-reaches-vars (typ vlist eqns)
  (catch 
      'reaches 
    (progn ()
	   (type-vars-reachable-main typ
				     eqns
				     #'(lambda (typ eqns as)
					 (declare (ignore eqns))
					 (if (and (is-var typ)
						  (member typ vlist :test #'var-=))
					     (throw 'reaches t)
					   (cons typ as)))
				     :fv-expand t
				     :bv-expand t)
	   t)
    nil))
					    
