(in-package 'types)

;;;  type-<= :: Eqns * bv-assumptions (bv <= X) * Type * Type -> (fn -> True/False)
;;;
;;; This file logically belongs with subset.lisp, but has to be physically
;;; separate to allow lisp to define things in the right order.
;;;



;;;
;;; The following 2-D array says which containment function should be applied.
;;; If testing t1 <= t2 and t1 has type i and t2 has type j, then apply
;;; the function in position (i,j).  Indexing is by rows and then columns.  
;;;
(defconstant *lessarray*
  (make-array 
   '(11 11)
   :initial-contents
(list 
;;                         union  intersection  cons   function  variable   bound var     not     dependent   value
;;        0         1         2        3         4          5         6          7         8          9         10
(list #'isless  #'isless #'isless  #'isless  #'isless   #'isless   #'isless  #'isless  #'isless   #'isless  #'isless );; 0
(list #'notless #'isless #'lessun  #'lessin  #'notless  #'notless  #'lessvar #'lessbv  #'notless  #'notless #'lessv  );; 1
(list #'uless   #'isless #'uless   #'uless   #'uless    #'uless    #'uless   #'uless   #'uless    #'uless   #'uless  );; 2 
(list #'inless  #'isless #'lessun  #'lessin  #'inless   #'inless   #'inless  #'inlessbv #'inless   #'notless #'lessv  );; 3 
(list #'notless #'isless #'lessun  #'lessin  #'consless #'notless  #'lessvar #'lessbv  #'conslnot #'notless #'lessv  );; 4 
(list #'notless #'isless #'lessun  #'lessin  #'notless  #'funcless #'lessvar #'lessbv  #'notless  #'notless #'lessv  );; 5  
(list #'varless #'isless #'lessun  #'lessin  #'varless  #'varless  #'varlvar #'lessbv  #'varless  #'varless #'lessv  );; 6
(list #'bvless  #'isless #'bvless  #'lessin  #'bvless   #'bvless   #'bvless #'bvlessbv #'bvless   #'bvless  #'lessv  );; 7
(list #'notless #'isless #'lessun  #'lessin  #'notless  #'notless  #'lessvar #'lessbv  #'nlessn   #'notless #'lessv  );; 8
(list #'notless #'isless #'lessun  #'lessin  #'depless  #'depless  #'depless #'lessbv  #'depless  #'dlessd  #'lessv  );; 9
(list #'notless #'isless #'lessun  #'lessin  #'vless    #'vless    #'lessvar #'lessbv  #'vless    #'notless #'lessv  );; 10
)))



;;;
;;; The main routine just applies the appropriate function 
;;; and updates statistics if required.
;;;	      
(defun type-<=-aux (t1 t2 Eqns as sols)
  (if (eq t1 t2)
      t
    (funcall (aref *lessarray* (type-type t1) (type-type t2)) t1 t2 Eqns as sols)))

;;;
;;; The top-level function is type-<=.  It just starts out the
;;; computation in the empty environment of assumptions and
;;; updates statistics.
;;;
(defun type-<= (t1 t2 Eqns &key (sols nil))
    (type-<=-aux t1 t2 Eqns (assump-empty) sols))
                                              
(defun type-= (t1 t2 eqns &key (sols nil))
  (and
   (type-<= t1 t2 eqns :sols sols)
   (type-<= t2 t1 eqns :sols sols)))
