(in-package 'types)

;;;
;;; type-tlv * eqns : type -> list-of-variables.
;;;
;;; tlv gets the top level variables in a type.
;;; note: the eqns argument is no longer needed, but
;;; is still here to avoid having to modify all uses of type-tlv.
 
;;; 0. tlv(0) = {}
;;;

(defun tlv-zero (typ eqns)
  (declare (ignore typ eqns))
  nil)

;;; 1. tlv(1) = {}
;;;

(defun tlv-one (typ eqns)
  (declare (ignore typ eqns))
  nil)

;;; 2. tlv(X | Y) = tlv(X) union tlv(Y)
;;;

(defun tlv-union (typ eqns)
  (append (type-tlv-aux (type-t1 typ) eqns) (type-tlv-aux (type-t2 typ) eqns)))

;;; 3. tlv(X * Y) = tlv(X) union tlv(Y)
;;;

(defun tlv-inter (typ eqns)
  (append (type-tlv-aux (type-t1 typ) eqns) (type-tlv-aux (type-t2 typ) eqns)))

;;; 4. tlv(c(X1,....Xn)) = {}
;;;

(defun tlv-cons (typ eqns)
  (declare (ignore typ eqns))
  nil)

;;; 5. tlv(A -> B + C) = {}
;;;

(defun tlv-func (typ eqns)
  (declare (ignore typ eqns))
  nil)

;;; 6. tlv(a) = {a} if a is a free variable
;;;

(defun tlv-var (typ eqns)
  (declare (ignore eqns))
  (list typ))

;;; 7. tlv(a) = {a} if a is a bound variable
;;;

(defun tlv-bvar (typ eqns)
  (declare (ignore eqns))
  (list typ ))

;;; 8. tlv(not(...)) = {}
;;;

(defun tlv-not (typ eqns)
  (declare (ignore typ eqns))
  nil)

;;; 9. tlv(A?B) = tlv(A) | tlv(B)
;;;

(defun tlv-dep (typ eqns)
  (append (type-tlv-aux (type-t1 typ) eqns) 
	  (type-tlv-aux (type-t2 typ) eqns)))

;;; 10. tlv(val(v,A)) = tlv(A)
;;;

(defun tlv-value (typ eqns)
  (type-tlv-aux (value-typ typ) eqns))
