(in-package 'types)

;;
;; A variable set abstraction
;;

(proclaim '(inline varset-empty varset-empty-p varset-adjoin
		   varset-intersection varset-union varset-member
		   varset-difference list-to-varset))

(defconstant varset-empty nil)

(defun varset-empty-p (x) (null x))

(defun varset-adjoin (x y) (ptools::sorted-adjoin x y #'var-<= #'var-=))

(defmacro varset-adjoin+ (x y)
  `(setq ,y (varset-adjoin ,x ,y)))

(defun varset-intersection (x y) (ptools::sorted-intersection x y #'var-<= #'var-=))

(defun varset-union (x y) (ptools::merge-uniquely x y #'var-<= #'var-=))

(defun varset-union-list (l) (reduce #'varset-union l
					   :initial-value varset-empty))

(defun varset-union-n (&rest args) (varset-union-list args))

(defun varset-member (x y) (member x y :test #'var-=))

(defun varset-difference (x y) (ptools::sorted-diff x y #'var-<= #'var-=))

(defun list-to-varset (x) (ptools::uniqify (sort (copy-list x) #'var-<=) #'var-=))

