(in-package 'types)

(defvar *source-type* (make-pathname :type "lisp"))
(defvar *binary-type* (make-pathname :type *object-file-extension*))
(defvar *load-type* (list (make-pathname :type *object-file-extension*)
			       (make-pathname :type "lisp")))

(defun default-pathname (path &rest more-paths)
  ;; builds first possible pathname out of defaulting paths, which may be
  ;; lists.
  (let ((res path))
    (dolist (defpath more-paths)
	    (if (listp defpath) (setq defpath (first defpath)))
	    (setq res (merge-pathnames res defpath)))
    res))

(defun first-exists-pathname (path &rest more-paths)
  ;; returns first defaulted pathname of an existing file.
  (let ((paths (list path)))
    (dolist (mp more-paths)
	    (if (not (listp mp)) (setq mp (list mp)))
	    (let ((sub-paths nil))
	      (dolist (p mp)
		      (dolist (sp paths)
			      (push (merge-pathnames sp p) sub-paths)))
	      (setq paths (nreverse sub-paths))))
    (dolist (path paths (first paths))
	    (if (probe-file path)
		(return path)))))

(defun compile&load (cvar lvar dir files)
  (dolist (file files)
	  (if cvar
	      (apply #'compile-file
		     (append (list (default-pathname dir file *source-type*)
				   :output-file
				   (default-pathname dir file *binary-type*)))))
	  (if lvar
	      (apply #'load
		     (list (first-exists-pathname dir file *load-type*))))))

(defun define-function (x y)
  (setf (symbol-function x) y))

(defun ensure-package (x &key (use '(lisp user)))
  (if (not (find-package x))
      (make-package x :use use)))
